import Environment from "./Environment.js";
import LogFmt from "./LogFmt.js";
import LogFormat from "./LogFormat.js";
import LogLevel from "./LogLevel.js";

export default class Log {

  static LOG_LEVEL;
  static LOG_FORMAT;

  static level(level = LogLevel.DEFAULT_LEVEL) {
    this.LOG_LEVEL = level;
    return this;
  }

  static format(format = LogFormat.DEFAULT_FORMAT) {
    this.LOG_FORMAT = format;
    return this;
  }

  static error(log, error = null) {
    this._initIfNeccesary();
    this._toStdErr(this._createCompleteLog(log, LogLevel.ERROR), error);
  }

  static warn(log, data = null) {
    this._initIfNeccesary();
    this._toStdErr(log, data);
  }

  static info(log, data = null) {
    this.log(log, data);
  }

  static log(log, data = null) {
    this._initIfNeccesary();



    this._toStdOut(`[INFO]: ${log}`, data);
  }

  static debug(log, data = null) {
    this._initIfNeccesary();
    if (!Environment.DEBUG()) return;
    this._toStdOut(`[DEBUG]: ${log}`, data);
  }

  static measure(log, data = null) {
    this.debug(log, data);
  }

  static trace(message, data = null) {
    this._initIfNeccesary();
    if (!Environment.DEBUG()) return;
    this._toStdOut(`[TRACE]: ${message}`, data);
  }

  static _initIfNeccesary() {
    if (!this.LOG_LEVEL) {
      this.LOG_LEVEL = LogLevel.fromEnv();
    }
    if (!this.LOG_FORMAT) {
      this.LOG_FORMAT = LogFormat.DEFAULT_FORMAT;
    }
  }

  static _createCompleteLog(log, level = LogLevel.DEFAULT_LEVEL) {
    if (this.LOG_FORMAT.isLogfmt()) {
      return LogFmt.addLevel(log, level);
    }
    return log;
  }

  static _toStdOut(log, data) {
    this._log(console.log, log, data);
  }

  static _toStdErr(log, data) {
    this._log(console.error, log, data);
  }

  static _log(logger, log, data) {
    logger(log);
    if (data) {
      logger(data);
    }
  }

}
